import functools
import sys
from os.path import expanduser
from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *

import creaturecast_designer.stylesheets as sst
import creaturecast_designer.widgets.loading_page as lpg
import creaturecast_designer.widgets.side_bar as sbr
import creaturecast_designer.widgets.login_widget as lgn
import creaturecast_designer.widgets.user_widget as uwd
import creaturecast_designer.widgets.user_header as uhd
import creaturecast_designer.widgets.icon as ico
import creaturecast_designer.widgets.connecting_widget as cwd
import creaturecast_designer.widgets.extensions_widget as exw
import creaturecast_designer.environment as env
dock_options = QMainWindow.AnimatedDocks | QMainWindow.AllowNestedDocks | QMainWindow.AllowTabbedDocks | QMainWindow.VerticalTabs
#| QMainWindow.AllowNestedDocks | QMainWindow.AllowTabbedDock

home_directory = expanduser("~").replace('\\', '/')
creaturecast_directory = '%screaturecast' % home_directory
extensions_directory = '%s/extensions' % creaturecast_directory
if extensions_directory not in sys.path:
    sys.path.append(extensions_directory)

class DockAreaWidget(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(DockAreaWidget, self).__init__(*args, **kwargs)


class DockStackWidget(QWidget):

    def __init__(self, *args, **kwargs):
        super(DockStackWidget, self).__init__(*args, **kwargs)
        self.user_widget = uwd.UserWidget(self)
        self.dock_area_widget = DockAreaWidget(self)
        self.extensions_widget = exw.ExtensionsWidget(self)

        self.dock_area_widget.setWindowFlags(Qt.Widget)
        self.stacked_layout = QStackedLayout(self)
        self.stacked_layout.addWidget(self.user_widget)
        self.stacked_layout.addWidget(self.dock_area_widget)
        self.stacked_layout.addWidget(self.extensions_widget)

        env.user_handler.user_changed.connect(self.set_user)

    def set_user(self, user_data):
        self.stacked_layout.setCurrentIndex(0)


class DesignerWidget(QWidget):

    item_font = QFont('', 12, False)

    def __init__(self, *args, **kwargs):
        super(DesignerWidget, self).__init__(*args, **kwargs)
        self.side_bar = sbr.SidebarWidget(self)
        self.dock_stack_widget = DockStackWidget(self)
        self.extensions_button = QPushButton(ico.get_icon('extensions'), 'Extensions', parent=self)
        self.extensions_button.setIconSize(QSize(25, 25))
        self.extensions_button.setFlat(True)
        self.settings_button = QPushButton(ico.get_icon('tool'), 'Settings', parent=self)
        self.settings_button.setIconSize(QSize(25, 25))
        self.settings_button.setFlat(True)
        #self.extensions_button.set_image(med.get_icon_path('extensions'))
        self.user_header = uhd.UserHeader(self)
        self.user_header.setMaximumWidth(215)
        self.user_header.setMinimumWidth(215)
        self.main_layout = QVBoxLayout(self)
        self.tools_layout = QHBoxLayout(self)
        self.body_layout = QHBoxLayout(self)
        self.side_bar_layout = QVBoxLayout(self)

        self.main_layout.addLayout(self.body_layout)
        self.main_layout.addLayout(self.tools_layout)
        self.tools_layout.addStretch()
        self.tools_layout.addWidget(self.extensions_button)
        self.tools_layout.addWidget(self.settings_button)
        self.tools_layout.addSpacerItem(QSpacerItem(32, 32))
        self.tools_layout.addStretch()
        self.body_layout.addLayout(self.side_bar_layout)
        self.side_bar_layout.addWidget(self.user_header)
        self.side_bar_layout.addWidget(self.side_bar)
        self.body_layout.addWidget(self.dock_stack_widget)

        self.side_bar.item_selection.connect(self.setup_widgets)
        self.user_header.clicked.connect(functools.partial(self.dock_stack_widget.stacked_layout.setCurrentIndex, 0))
        self.extensions_button.clicked.connect(self.show_extensions_widget)

        self.dock_widgets = []

        self.main_layout.setContentsMargins(5, 5, 5, 5)
        self.main_layout.setSpacing(5)
        self.tools_layout.setContentsMargins(0,0,0,0)
        self.tools_layout.setSpacing(0)
        self.body_layout.setContentsMargins(0,0,0,0)
        self.body_layout.setSpacing(0)

    def show_extensions_widget(self):
        self.dock_stack_widget.stacked_layout.setCurrentIndex(2)
        self.dock_stack_widget.extensions_widget.get_tools()


    def setup_widgets(self, items):

        for dock_widget in self.dock_widgets:
            dock_widget.setVisible(False)
        if items:
            self.dock_stack_widget.stacked_layout.setCurrentIndex(1)

        for item in items:
            if item.get('dock_widget', None):
                item['dock_widget'].setVisible(True)
            else:
                dock_area_widget = self.dock_stack_widget.dock_area_widget
                loading_page = lpg.LoadingPage(self)
                dock_widget = PaintedDock(item['name'], dock_area_widget)
                dock_widget.setWidget(loading_page)
                dock_widget.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
                dock_widget.setFont(self.item_font)
                dock_area_widget.addDockWidget(Qt.RightDockWidgetArea, dock_widget, Qt.Horizontal)
                item['dock_widget'] = dock_widget

                QApplication.processEvents()

                module_string = '.'.join(item['class_location'].split('.')[0:-1])
                class_string = item['class_location'].split('.')[-1]

                ''''
                if not 'C:/Users/User/creaturecast/modules' in sys.path:
                    raise Exception('No Sys path')
                import os
                if not os.path.exists('C:/Users/User/creaturecast/modules'):
                    raise Exception('No Modules path')
                if not module_string.split('.')[0] in os.listdir('C:/Users/User/creaturecast/modules'):
                    print 'No Module', module_string
                if not os.path.exists('%s/%s/__init__.py' %('C:/Users/User/creaturecast/modules', module_string.split('.')[0])):
                    print 'No __innit__ ', '%s/%s/__init__.py' %('C:/Users/User/creaturecast/modules', module_string.split('.')[0])

                print module_string
                '''
                module = __import__(module_string, fromlist=['.'])

                widget = module.__dict__[class_string](self)

                dock_widget.setWidget(widget)
                self.dock_widgets.append(dock_widget)

                if hasattr(widget, 'nodes_selected'):
                    if isinstance(widget.nodes_selected, Signal):
                        widget.nodes_selected.connect(self.nodes_selected.emit)

                if hasattr(widget, 'add_nodes'):
                    self.nodes_selected.connect(widget.add_nodes)



class DesignerStackWidget(QWidget):

    def __init__(self, *args, **kwargs):
        super(DesignerStackWidget, self).__init__(*args, **kwargs)

        self.login_widget = lgn.LoginWidget(self)
        self.designer_widget = DesignerWidget(self)
        self.connecting_widget = cwd.ConnectingWidget(self)
        self.stacked_layout = QStackedLayout(self)

        self.stacked_layout.addWidget(self.login_widget)
        self.stacked_layout.addWidget(self.designer_widget)
        self.stacked_layout.addWidget(self.connecting_widget)

        self.login_widget.start_login.connect(self.start_login)
        env.user_handler.user_changed.connect(self.set_user)
        self.designer_widget.dock_stack_widget.user_widget.log_out.connect(self.logged_out)
        self.connecting_widget.canceled.connect(self.logged_out)

    def start_login(self, *args, **kwargs):
        self.stacked_layout.setCurrentIndex(2)
        self.connecting_widget.start_animation()

    def logged_out(self, *args, **kwargs):
        self.stacked_layout.setCurrentIndex(0)

    def set_user(self, user_data):
        self.stacked_layout.setCurrentIndex(1)



class DesignerWindow(QMainWindow):
    item_font = QFont('', 12, False)
    nodes_selected = Signal(list)

    def __init__(self, *args, **kwargs):
        super(DesignerWindow, self).__init__(*args, **kwargs)
        self.setWindowTitle('Creaturecast Designer')
        self.central_widget = DesignerStackWidget(self)
        self.setCentralWidget(self.central_widget)
        self.setProperty('stylesheet_preset', 'main_window')





class ToolButton(QLabel):

    clicked = Signal()

    def __init__(self, *args, **kwargs):
        super(ToolButton, self).__init__(*args, **kwargs)
        self.image_size = QSize(32, 32)
        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)

    def mousePressEvent(self, event):
        self.clicked.emit()

    def set_image_size(self, size):
        self.image_size = size

    def sizeHint(self, *args, **kwargs):
        return self.image_size

    def set_image(self, path):
        height = self.image_size.height()
        width = self.image_size.width()

        user_image = QImage(path)

        user_image = user_image.scaled(
            height,
            width,
            Qt.IgnoreAspectRatio,
            Qt.SmoothTransformation
        )

        user_pixmap = QPixmap.fromImage(user_image)

        self.setPixmap(user_pixmap)




class PaintedDock(QDockWidget):
    def __init__(self, *args, **kwargs):
        super(PaintedDock, self).__init__(*args, **kwargs)



'''
class DesignerWidget(QWidget):
    item_font = QFont('', 12, False)
    nodes_selected = Signal(list)

    def __init__(self, *args, **kwargs):
        super(DesignerWidget, self).__init__(*args, **kwargs)

        #Create Widgets


        #Create Layouts

        self.stacked_layout = QStackedLayout(self)
        self.horizontal_layout = QHBoxLayout()

        #Connect Layouts

        self.stacked_layout.addWidget(self.log_in_widget)
        self.stacked_layout.ad(self.)
        self.horizontal_layout.addWidget(self.side_bar)
        self.horizontal_layout.addWidget(self.stacked_widget)
        #self.main_stack_layout.addWidget(self.dock_area_widget)

        #Signals
        self.log_in_widget.logged_in.connect(self.set_user)
        self.user_widget.log_out.connect(self.logged_out)
        #self.side_bar.item_selection.connect(self.setup_widgets)

        self.dock_widgets = []


        self.stacked_layout_2 = QStackedLayout(self.stacked_widget)

        self.dock_layout = QHBoxLayout(self.dock_area)
        self.dock_layout.addWidget(self.dock_central_widget)


        #Connect Layouts

        self.stacked_layout.addWidget(self.main_widget)
        self.stacked_layout.addWidget(self.log_in_widget)
        self.stacked_layout.setCurrentIndex(1)
        self.horizontal_layout.addWidget(self.side_bar)
        self.horizontal_layout.addWidget(self.stacked_widget)
        self.stacked_layout_2.addWidget(self.user_widget)
        self.stacked_layout_2.addWidget(self.dock_area)



        #Set Properties
        self.setCentralWidget(self.central_widget)


        self.setContentsMargins(0, 0, 0, 0)
        self.horizontal_layout.setContentsMargins(0, 0, 0, 0)
        self.horizontal_layout.setContentsMargins(0, 0, 0, 0)
        self.stacked_layout.setContentsMargins(5, 10, 0, 5)
        self.dock_widgets = []

        #Signals
        self.side_bar.log_out.connect(self.logged_out)


    def set_user(self, user_data):
        self.stacked_layout.setCurrentIndex(1)
        self.main_stack_layout.setCurrentIndex(0)

        self.side_bar.user_widget.set_user(user_data)
        self.side_bar.load_tools(user_data)
        self.user_widget.set_user(user_data)


    def logged_out(self, *args, **kwargs):
        self.stacked_layout.setCurrentIndex(0)



    def setup_widgets(self, items):

        for dock_widget in self.dock_widgets:
            dock_widget.setVisible(False)
        if items:
            self.main_stack_layout.setCurrentIndex(1)

        for item in items:
            if item.get('dock_widget', None):
                item['dock_widget'].setVisible(True)
            else:
                loading_page = lpg.LoadingPage(self)
                dock_widget = PaintedDock(item['name'], self.dock_area_widget)
                dock_widget.setWidget(loading_page)
                dock_widget.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
                dock_widget.setFont(self.item_font)
                self.dock_area_widget.addDockWidget(Qt.RightDockWidgetArea, dock_widget, Qt.Horizontal)
                item['dock_widget'] = dock_widget

                QApplication.processEvents()

                module_string = '.'.join(item['class_location'].split('.')[0:-1])
                class_string = item['class_location'].split('.')[-1]

                if not 'C:/Users/User/creaturecast/modules' in sys.path:
                    raise Exception('No Sys path')
                import os
                if not os.path.exists('C:/Users/User/creaturecast/modules'):
                    raise Exception('No Modules path')
                if not module_string.split('.')[0] in os.listdir('C:/Users/User/creaturecast/modules'):
                    print 'No Module', module_string
                if not os.path.exists('%s/%s/__init__.py' %('C:/Users/User/creaturecast/modules', module_string.split('.')[0])):
                    print 'No __innit__ ', '%s/%s/__init__.py' %('C:/Users/User/creaturecast/modules', module_string.split('.')[0])

                print module_string

                print 'Importing %s...' % class_string, module_string
                module = __import__(module_string, fromlist=['.'])
                print 'Instanciating %s.%s.' % (module_string, class_string)

                widget = module.__dict__[class_string](self)
                print 'Dockingating %s...' % class_string

                dock_widget.setWidget(widget)
                self.dock_widgets.append(dock_widget)


                if hasattr(widget, 'nodes_selected'):
                    if isinstance(widget.nodes_selected, Signal):
                        widget.nodes_selected.connect(self.nodes_selected.emit)

                if hasattr(widget, 'add_nodes'):
                    self.nodes_selected.connect(widget.add_nodes)

                if hasattr(widget, 'go_to_tool'):
                    if isinstance(widget.go_to_tool, Signal):
                        widget.go_to_tool.connect(self.go_to_tool)

                if hasattr(widget, 'new_tool'):
                    if isinstance(widget.new_tool, Signal):
                        widget.new_tool.connect(self.side_bar.build_model)

                if hasattr(widget, 'delete_tool'):
                    if isinstance(widget.delete_tool, Signal):
                        widget.delete_tool.connect(self.side_bar.build_model)

    def go_to_tool(self, table_item):
        model = self.side_bar.list_view.model()
        for row, item in enumerate(model.items):
            if item.class_location == table_item.class_location:
                index = model.index(row, 0, QModelIndex())
                self.side_bar.list_view.clearSelection()
                self.side_bar.list_view.selectionModel().select(index, QItemSelectionModel.Select)
'''


def run():
    import sys
    app = QApplication(sys.argv)
    app.setStyleSheet(sst.get_stylesheet('slate'))
    mainWin = DesignerWindow()
    mainWin.show()
    sys.exit(app.exec_())

if __name__ == '__main__':

    run()


from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *

class Window(QWidget):
    def __init__(self):
        super(Window, self).__init__()

        flowLayout = FlowLayout()
        flowLayout.addWidget(MyWidget(self))
        flowLayout.addWidget(MyWidget(self))
        flowLayout.addWidget(MyWidget(self))
        flowLayout.addWidget(MyWidget(self))
        flowLayout.addWidget(MyWidget(self))
        self.setLayout(flowLayout)
        self.setWindowTitle("Flow Layout")


class FlowLayout(QLayout):

    geometry_changed = Signal()

    def __init__(self, parent=None, margin=0, spacing=-1):
        super(FlowLayout, self).__init__(parent)

        #if parent is not None:
        #    self.setMargin(margin)

        self.setSpacing(spacing)

        self.itemList = []

    def __del__(self):
        item = self.takeAt(0)
        while item:
            item = self.takeAt(0)

    def addItem(self, item):
        self.itemList.append(item)

    def count(self):
        return len(self.itemList)

    def itemAt(self, index):
        if index >= 0 and index < len(self.itemList):
            return self.itemList[index]

        return None

    def takeAt(self, index):
        if index >= 0 and index < len(self.itemList):
            return self.itemList.pop(index)

        return None

    #def expandingDirections(self):
    #    return Qt.Orientations(Qt.Orientation(0))

    #def hasHeightForWidth(self):
    #    return True

    #def heightForWidth(self, width):
    #    height = self.doLayout(QRect(0, 0, width, 0), True)
    #    return height

    def setGeometry(self, rect):
        super(FlowLayout, self).setGeometry(rect)
        self.doLayout(rect, False)
        self.geometry_changed.emit()

    def sizeHint(self):
        return self.minimumSize()

    def minimumSize(self):
        size = QSize()
        for item in self.itemList:
            size = size.expandedTo(item.minimumSize())
        size += QSize(2 * self.margin(), 2 * self.margin())
        return size

    def doLayout(self, rect, testOnly):
        x = rect.x()
        y = rect.y()
        lineHeight = 0

        for item in self.itemList:
            wid = item.widget()
            spaceX = self.spacing() + wid.style().layoutSpacing(QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Horizontal)
            spaceY = self.spacing() + wid.style().layoutSpacing(QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Vertical)
            nextX = x + item.sizeHint().width() + spaceX
            if nextX - spaceX > rect.right() and lineHeight > 0:
                x = rect.x()
                y = y + lineHeight + spaceY
                nextX = x + item.sizeHint().width() + spaceX
                lineHeight = 0

            if not testOnly:
                item.setGeometry(QRect(QPoint(x, y), item.sizeHint()))

            x = nextX
            lineHeight = max(lineHeight, item.sizeHint().height())

        return y + lineHeight - rect.y()


class MyWidget(QWidget):
    clicked = Signal()
    keyPressed = Signal(unicode)

    def __init__(self, *args, **kwargs):
        QWidget.__init__(self, *args, **kwargs)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHeightForWidth(True)
        self.setSizePolicy(sizePolicy)

    def sizeHint(self):
        return QSize(128, 128)

    def heightForWidth(self, width):
        return width * 1.5




if __name__ == '__main__':

    import sys

    app = QApplication(sys.argv)

    mainWin = Window()
    mainWin.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

"""
  Author: Jared P. Sutton <jpsutton@gmail.com>
  License: LGPL
  Note: I've licensed this code as LGPL because it was a complete translation of the code found here...
    https://github.com/mojocorp/QProgressIndicator
    
"""

import sys
from PySide.QtCore import *
from PySide.QtGui import *

class ProgressWidget (QWidget):
  m_angle = None
  m_timerId = None
  m_delay = None
  m_displayedWhenStopped = None
  m_color = None

  def __init__ (self, parent):
    # Call parent class constructor first
    super(ProgressWidget, self).__init__(parent)
    
    # Initialize Qt Properties
    self.setProperties()
    
    # Intialize instance variables
    self.m_angle = 0
    self.m_timerId = -1
    self.m_delay = 40
    self.m_displayedWhenStopped = False
    self.m_color = Qt.white
    
    # Set size and focus policy
    self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
    self.setFocusPolicy(Qt.NoFocus)
    self.background_color = QColor(100, 100, 100, 200)
    # Show the widget
    self.show()
    
  def animationDelay (self):
    return self.delay
    
  def isAnimated (self):
    return (self.m_timerId != -1)
    
  def isDisplayedWhenStopped (self):
    return self.displayedWhenStopped
    
  def getColor (self):
    return self.color
    
  def sizeHint (self):
    return QSize(20, 20)
  
  def startAnimation (self):
    self.m_angle = 0
    
    if self.m_timerId == -1:
      self.m_timerId = self.startTimer(self.m_delay)
    
  def stopAnimation (self):
    if self.m_timerId != -1:
      self.killTimer(self.m_timerId)
      
    self.m_timerId = -1
    self.update()
    
  def setAnimationDelay (self, delay):
    if self.m_timerId != -1:
      self.killTimer(self.m_timerId)
      
    self.m_delay = delay
    
    if self.m_timerId != -1:
      self.m_timerId = self.startTimer(self.m_delay)
    
  def setDisplayedWhenStopped (self, state):
    self.displayedWhenStopped = state
    self.update()
    
  def setColor (self, color):
    self.m_color = color
    self.update()
    
  def timerEvent (self, event):
    self.m_angle = (self.m_angle + 30) % 360
    self.update()
    
  def paintEvent(self, event):


    if (not self.m_displayedWhenStopped) and (not self.isAnimated()):
      return

    width = min(self.width()*0.5, self.height()*0.5)
    painter = QPainter(self)
    painter.setRenderHint(QPainter.Antialiasing)

    painter.fillRect(self.rect(), self.background_color)
    outerRadius = (width - 1) * 0.25
    innerRadius = (width - 1) * 0.25 * 0.38
    
    capsuleHeight = outerRadius - innerRadius
    capsuleWidth  = capsuleHeight *.23 if (width > 32) else capsuleHeight *.35
    capsuleRadius = capsuleWidth / 2
    
    for i in range(0, 12):
      color = QColor(self.m_color)

      if self.isAnimated():
        color.setAlphaF(1.0 - (i / 12.0))
      else:
        color.setAlphaF(0.2)

      painter.setPen(Qt.NoPen)
      painter.setBrush(color)
      painter.save()
      painter.translate(self.rect().center())
      painter.rotate(self.m_angle - (i * 30.0))
      painter.drawRoundedRect(capsuleWidth * -0.5, (innerRadius + capsuleHeight) * -1, capsuleWidth, capsuleHeight, capsuleRadius, capsuleRadius)
      painter.restore()
      
  def setProperties (self):
    self.delay = Property(int, self.animationDelay, self.setAnimationDelay)
    self.displayedWhenStopped = Property(bool, self.isDisplayedWhenStopped, self.setDisplayedWhenStopped)
    self.color = Property(QColor, self.getColor, self.setColor)


def TestProgressIndicator ():
  app = QApplication(sys.argv)
  progress = ProgressWidget(None)
  progress.setAnimationDelay(70)
  progress.startAnimation()

  # Execute the application
  sys.exit(app.exec_())
  
if __name__ == "__main__":
  TestProgressIndicator()

  



import json
import time
import creaturecast_designer.services as svc
import requests
url=svc.library_url


def get_user_data(token, pause=0.1):
    while True:
        request = requests.get(
            '%s/get_user_data' % url,
            params=dict(
                token=token
            )
        )
        if request.ok:
            json_string = request.text
            response_data = json.loads(json_string)
            if response_data['status'] == 'succeeded':
                return response_data['data']
            else:
                print response_data['message']
                time.sleep(pause)
        else:
            print request.text
            print 'Failed with error code %s' % request.status_code
            return



if __name__ == '__main__':
    print get_user_data('Foo')



import copy
from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *


class Port(QGraphicsWidget):

    size = 12

    def __init__(self, *args, **kwargs):
        super(Port, self).__init__(*args, **kwargs)

        self.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
        self.setPreferredSize(QSize(self.size, self.size))
        self.setWindowFrameMargins(1, 1, 1, 1)
        self.transform().translate(0, 0)
        self.default_pen = QPen(QColor(200, 100, 100), 1.0)
        self.hover_pen = QPen(QColor(200, 100, 100), 1.5)
        self.brush = QBrush(QColor(200, 100, 100))
        polygon = QPolygonF([QPoint(*x) for x in [0, 0], [0, self.size], [self.size*0.5, self.size*0.5], [0, 0]])
        self.elipse_item = QGraphicsPolygonItem(polygon, parent=self)
        self.elipse_item.setPen(self.default_pen)
        self.elipse_item.setBrush(self.brush)
        #self.elipse_item.setPos(size.width()/2, size.height()/2)
        #self.elipse_item.setRect(
        #    -self.radius,
        #    -self.radius,
        #    self.diameter,
        #    self.diameter,
        #    )


        #self.setColor(QColor(255, 0, 0))
        self.setAcceptHoverEvents(True)


    #def paint(self, painter, option, widget):
    #    window_rect = self.windowFrameRect()

    #    painter.setBrush(QColor(100, 100, 100))
    #
    #    painter.drawRect(window_rect)


class Attribute(QGraphicsWidget):
    font = QFont("", 20, True)
    font.setWeight(0)


    def __init__(self, text, parent):
        super(Attribute, self).__init__(parent)
        self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.setPreferredSize(QSize(100, 20))
        layout = QGraphicsLinearLayout(self)
        layout.setOrientation(Qt.Horizontal)
        layout.setContentsMargins(2, 2, 2, 2)
        layout.setSpacing(3)

        #self.text_widget = QGraphicsWidget(self)
        #self.text_item = QGraphicsTextItem(text, self.text_widget)
        #self.text_item.setDefaultTextColor(parent.outline_color)
        #self.text_item.setFont(self.font)
        #option = self.text_item.document().defaultTextOption()
        #option.setWrapMode(QTextOption.NoWrap)
        #self.text_item.document().setDefaultTextOption(option)
        #self.text_item.adjustSize()


        self.in_port = Port()
        self.out_port = Port()

        layout.addItem(self.in_port)
        layout.setAlignment(self.in_port, Qt.AlignLeft| Qt.AlignTop)

        layout.addStretch()
        #layout.addItem(self.text_widget)
        #layout.setAlignment(self.text_widget, Qt.AlignCenter| Qt.AlignTop)

        layout.addStretch()
        layout.addItem(self.out_port)
        layout.setAlignment(self.out_port, Qt.AlignRight| Qt.AlignTop)

        self.setLayout(layout)

    def get_text_size(self):
        return QSizeF(
            self.text_item.textWidth(),
            self.font.pointSizeF()
            )




class NodeTitle(QGraphicsWidget):

    font = QFont("arial", 32, True)
    font.setWeight(0)
    font.setLetterSpacing(QFont.PercentageSpacing, 100)

    def __init__(self, text, parent):
        super(NodeTitle, self).__init__(parent)

        self.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
        layout = QGraphicsLinearLayout(self)
        layout.setOrientation(Qt.Horizontal)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        self.text_item = QGraphicsTextItem(text, self)

        self.text_item.setDefaultTextColor(parent.color)
        self.text_item.setFont(self.font)
        option = self.text_item.document().defaultTextOption()
        option.setWrapMode(QTextOption.NoWrap)
        self.text_item.document().setDefaultTextOption(option)
        #self.text_item.adjustSize()
        self.setPreferredSize(QSize(100, 10))
        layout.addItem(self.text_item)
        self.setLayout(layout)

    def set_text(self, text):
        self.text_item.setPlainText(text)
        self.text_item.adjustSize()
        self.setPreferredSize(self.textSize())

    def get_text_size(self):
        return QSizeF(
            self.text_item.textWidth(),
            self.font.pointSizeF()
            )


    def paint(self, painter, option, widget):
        window_rect = self.windowFrameRect()
        # window_rect.setLeft(20)


        painter.drawRoundedRect(window_rect, 5, 5)


class NodeHeader(QGraphicsWidget):

    icon_size = QSize(42, 42)
    def __init__(self, text, *args, **kwargs):
        super(NodeHeader, self).__init__(*args, **kwargs)
        self.icon='handle_blackWhite'
        self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        layout = QGraphicsLinearLayout(self)
        layout.setOrientation(Qt.Horizontal)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        pixmap = QPixmap(
            "/Users/paxtongerrish/Scripts/creaturecast_pyside/media/icons/%s.png"
            % self.icon).scaled(self.icon_size, transformationMode=Qt.SmoothTransformation )
        self.icon_widget = QGraphicsWidget(self)
        QGraphicsPixmapItem(pixmap, self.icon_widget)
        self.icon_widget.setMaximumSize(self.icon_size)

        self.color = QColor(120, 120, 120)
        #self.title_widget = NodeTitle(text, self)
        layout.addItem(self.icon_widget)
        #layout.addItem(self.title_widget)
        self.setLayout(layout)

    def setText(self, text):
        self.title_widget.setText(text)





class Node(QGraphicsWidget):


    def __init__(self, graph, name='', icon='chain'):
        super(Node, self).__init__()
        self.setMinimumWidth(100)
        self.setMinimumHeight(50)
        self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        layout = QGraphicsLinearLayout(self)
        layout.setOrientation(Qt.Vertical)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(2)
        self.name = name
        self.icon = icon
        self.dragging = True
        self.mouse_start = QPoint()
        self.mouse_delta = QPoint()
        self.last_drag_point = QPoint()
        self.nodes_moved = False
        self.color = QColor(40, 40, 40)
        self.outline_color = QColor(140, 140, 140)
        self.selected_color = self.color

        self.outline_pen = QPen(self.outline_color, 1)
        self.outline_selected_pen = QPen(self.outline_color.lighter(80), 1)
        self.outline_selected_pen.setStyle(Qt.DashLine)

        self.selected = False
        self.dragging = False
        self.graph = graph

        self.icon = NodeHeader(text=self.name, parent=self)
        self.icon.setTransform(QTransform.fromTranslate(-32, -32))

        self.text_item = QGraphicsTextItem(name, self)
        self.text_item.setDefaultTextColor(self.outline_color)
        self.text_item.setFont(QFont('arial', 32))
        rect = self.text_item.boundingRect()
        self.text_item.setPos(QPoint(0, 0))



        for attribute_name in ['parent', 'spaces', 'size', 'side', 'index', 'color']:
            new_attribute = Attribute(attribute_name, self)
            layout.addItem(new_attribute)
            layout.setAlignment(new_attribute, Qt.AlignCenter | Qt.AlignTop)


        self.graph.scene().addItem(self)
        self.graph.selected_nodes.append(self)

        self.setLayout(layout)

    def paint(self, painter, option, widget):
        window_rect = self.windowFrameRect()
        #window_rect.setLeft(20)
        #window_rect.setBottom(window_rect.bottom()+30)
        if self.selected:
            painter.setBrush(self.color)
            painter.setPen(self.outline_selected_pen)

        else:
            painter.setBrush(self.selected_color)
            painter.setPen(self.outline_pen)

        painter.drawRoundedRect(window_rect, 5, 5)

    def get_translation(self):
        transform = self.transform()
        size = self.size()
        return QPointF(transform.dx()+(size.width()*0.5), transform.dy()+(size.height()*0.5))


    def set_translation(self, graphPos):
        size = self.size()
        self.setTransform(QTransform.fromTranslate(graphPos.x()-(size.width()*0.5), graphPos.y()-(size.height()*0.5)), False)

    def translate(self, x, y):
        super(Node, self).moveBy(x, y)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            modifiers = event.modifiers()
            if modifiers == Qt.ShiftModifier:
                self.selected = not self.selected
            else:
                self.selected = True
            self.dragging = True
            self.mouse_start = self.mapToScene(event.pos())
            self.mouse_delta = self.mouse_start - self.get_translation()
            self.last_drag_point = self.mouse_start
            self.nodes_moved = False
            self.update()

        else:
            super(Node, self).mousePressEvent(event)

    def mouseMoveEvent(self, event):
        if self.dragging:
            mouse_down_position = self.mapToScene(event.pos())
            delta = mouse_down_position - self.last_drag_point
            #for node in self.graph.selected_nodes:
            self.translate(delta.x(), delta.y())
            self.last_drag_point = mouse_down_position
            self.nodes_moved = True
        else:
            super(Node, self).mouseMoveEvent(event)


    def mouseReleaseEvent(self, event):
        if self.dragging:
            if self.nodes_moved:
                new_position = self.mapToScene(event.pos())
                delta = new_position - self.mouse_start
            self.setCursor(Qt.ArrowCursor)
            self.dragging = False
        else:
            super(Node, self).mouseReleaseEvent(event)


from qtpy import QtGui, QtWidgets, QtCore


class SelectionRect(QtWidgets.QGraphicsWidget):
    __backgroundColor = QtGui.QColor(100, 100, 100, 50)
    __pen = QtGui.QPen(QtGui.QColor(25, 25, 25), 1.0,  QtCore.Qt.DashLine)

    def __init__(self, graph, mouseDownPos):
        super(SelectionRect, self).__init__()
        self.setZValue(-1)

        self.__graph = graph
        self.__graph.scene().addItem(self)
        self.__mouseDownPos = mouseDownPos
        self.setPos(self.__mouseDownPos)
        self.resize(0, 0)

    def setDragPoint(self, dragPoint):
        topLeft = QtCore.QPointF(self.__mouseDownPos)
        bottomRight = QtCore.QPointF(dragPoint)
        if dragPoint.x() < self.__mouseDownPos.x():
            topLeft.setX(dragPoint.x())
            bottomRight.setX(self.__mouseDownPos.x())
        if dragPoint.y() < self.__mouseDownPos.y():
            topLeft.setY(dragPoint.y())
            bottomRight.setY(self.__mouseDownPos.y())
        self.setPos(topLeft)
        self.resize(bottomRight.x() - topLeft.x(), bottomRight.y() - topLeft.y())


    def paint(self, painter, option, widget):
        rect = self.windowFrameRect()
        painter.setBrush(self.__backgroundColor)
        painter.setPen(self.__pen)
        painter.drawRect(rect)

    def destroy(self):
        self.__graph.scene().removeItem(self)



from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *


class AbstractPropertiesModel(QAbstractTableModel):

    def __init__(self, **properties):
        super(AbstractPropertiesModel, self).__init__()
        self.key_font = QFont('None', 14, True)
        self.value_font = QFont('None', 12, False)
        self.key_color = QColor(111, 111, 111)
        self.value_background_color = QColor(210, 210, 210)
        self.value_color = QColor(22, 22, 22)

        self.properties = properties
        self.keys = properties.keys()

    def __setitem__(self, key, value):
        if key in self.keys:
            self.properties[key] = value
            index_row = self.keys.index(key)
            index = self.index(index_row, 0, QModelIndex())
            self.dataChanged.emit(index, index, 0)

    def value_from_index(self, index):
        row = index.row()
        if row != -1:
            return self.properties[self.keys[row]]

    def rowCount(self, index):
        return len(self.keys)

    def columnCount(self, index):
        return 2

    def data(self, index, role):
        column = index.column()
        key = self.keys[index.row()]
        value = self.value_from_index(index)

        if column == 0:
            if role == Qt.FontRole:
                return self.key_font
            if role == Qt.BackgroundColorRole:
                return self.key_color
            if role == Qt.TextAlignmentRole:
                return Qt.AlignLeft
            if role == Qt.DisplayRole or role == Qt.EditRole:
                return key

        if column == 1:
            if role == Qt.FontRole:
                return self.value_font
            if role == Qt.BackgroundColorRole:
                return self.value_background_color
            if role == Qt.TextColorRole:
                return self.value_color
            if role == Qt.TextAlignmentRole:
                return Qt.AlignLeft
            if role == Qt.DisplayRole:
                if isinstance(value, (list, tuple, dict)):
                    value = str(type(value))
                return value

            if role == Qt.EditRole:
                return str(value)


    def flags(self, index):
        column = index.column()
        if column == 0:
            return Qt.ItemIsEnabled
        if column == 1:
            return Qt.ItemIsEnabled | Qt.ItemIsDropEnabled | Qt.ItemIsDragEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable

    def setData(self, index, value, role=Qt.EditRole):
        row = index.row()
        column = index.column()
        key = self.keys[index.row()]
        if role == Qt.EditRole:
            current_value = self.value_from_index(index)
            if isinstance(current_value, (list, tuple, dict)):
                value = eval(value)
            self.properties[key] = value
            self.dataChanged.emit(index, index, 0)
            return True
        return False


class PropertiesModel(AbstractPropertiesModel):

    def __init__(self, **properties):
        super(PropertiesModel, self).__init__(**properties)



import os
from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *

fonts_directory = os.path.dirname(__file__.replace('\\', '/'))

font_cache = dict()

def get_font(*args):
    if args[0] in font_cache:
        return font_cache[args[0]]
    for item in os.listdir(fonts_directory):
        if item == '%s.otf' % args[0]:
            QFontDatabase.addApplicationFont('%s/%s' % (fonts_directory, item))
            font = QFont(*args)
            font_cache[args[0]] = font
            return font


from contextlib import closing
import requests
import creaturecast_designer.services as svc

chunk_size = 24


def list_extensions():
    list_tools_url = '%s/list_extensions' % svc.library_url
    response = requests.get(list_tools_url, stream=True)
    if response.ok:
        with closing(response) as r:
            for line in svc.split_by_lines(r.iter_content(chunk_size)):
                if line:
                    yield line
    else:
        raise Exception(response.text)


if __name__ == '__main__':
    for x in list_extensions():
        print x



import functools
import os
import shutil
import copy
import json
from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *
import PySignal

from creaturecast_designer.widgets.progress_widget import ProgressWidget
import creaturecast_designer.widgets.flow_layout as flt
import creaturecast_designer.environment as env
import creaturecast_designer.widgets.icon as ico
import creaturecast_designer.services.list_extensions as lex
from creaturecast_designer.services.thread_worker import GeneratorWorker, FunctionWorker
import creaturecast_designer.media as med
import creaturecast_designer.services.pip_functions as pfn
import creaturecast_designer.services.update_user_data as uud

item_size = 1.0
library_url = 'https://creaturecast-library.herokuapp.com'


class ExtensionsWidget(QFrame):

    go_to_tool = Signal(dict)
    delete_tool = Signal(dict)

    def __init__(self, *args, **kwargs):
        super(ExtensionsWidget, self).__init__(*args, **kwargs)
        self.main_layout = QVBoxLayout(self)
        self.search_layout = QHBoxLayout()
        self.search_button = QPushButton('Search', self)

        self.search_field = SearchLineEdit(self)
        self.scroll_area = QScrollArea(self)
        self.search_field.setFont(QFont('', 12, False))
        self.search_button.setFont(QFont('', 12, False))
        self.main_layout.addLayout(self.search_layout)
        self.search_layout.addWidget(self.search_field)
        self.search_layout.addWidget(self.search_button)

        self.main_layout.addWidget(self.scroll_area)
        self.flow_layout = flt.FlowLayout(self.scroll_area)
        self.setAcceptDrops(True)
        self.main_layout.setSpacing(5)
        self.main_layout.setContentsMargins(10, 5, 5, 5)
        self.flow_layout.setSpacing(8)
        self.flow_layout.setContentsMargins(0, 0, 0, 0)
        self.search_layout.setSpacing(5)
        self.search_layout.setContentsMargins(0, 0, 0, 0)


        self.busy_widget = ProgressWidget(self)
        self.busy_widget.setAnimationDelay(70)
        self.busy_widget.setVisible(False)
        self.search_button.pressed.connect(self.get_tools)


    def get_tools(self):

        search_text = self.search_field.text()
        self.busy_widget.setVisible(True)
        self.busy_widget.startAnimation()
        self.tools_worker = GeneratorWorker(lex.list_extensions)
        self.tools_thread = QThread()
        self.tools_thread.start()
        self.tools_worker.moveToThread(self.tools_thread)
        self.tools_worker.data.connect(self.create_item)
        self.tools_worker.success.connect(self.finish_loading)
        self.tools_worker.start.connect(self.tools_worker.run)
        self.tools_worker.start.emit()

    def finish_loading(self):
        self.busy_widget.setVisible(False)

    def create_item(self, json_string):
        data = json.loads(json_string)
        tool_widget = ExtensionWidget(data, self)
        self.flow_layout.addWidget(tool_widget)
        QApplication.processEvents()

    def search_items(self, text):
        for item in self.flow_layout.itemList:
            widget = item.widget()
            widget.setVisible(True)
            if not text.lower() in widget.table_object.name.lower():
                widget.setVisible(False)


    def resizeEvent(self, event):
        self.busy_widget.resize(event.size())
        event.accept()


class SearchLineEdit(QLineEdit):
    def __init__(self, *args, **kwargs):
        super(SearchLineEdit, self).__init__(*args, **kwargs)

        self.setPlaceholderText(self.tr('Search'))
        font = QFont('', 12, False)
        self.setFont(font)
        self.setCursor(Qt.ArrowCursor)

        self.button_toggle = QPushButton(self)
        self.button_toggle.setIcon(ico.get_icon('x'))
        self.button_toggle.setFlat(True)
        self.button_toggle.setIconSize(QSize(22, 22))
        self.button_toggle.setStyleSheet('QPushButton { border: none; padding: 0px;}')

        frame_width = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
        #self.setStyleSheet('QLineEdit {{ padding-right: {}px; }} '.format(self.button_toggle.sizeHint().width() + frame_width + 1))
        msz = self.minimumSizeHint()
        self.setMinimumSize(max(msz.width(), self.button_toggle.sizeHint().height() + frame_width * 2 + 2),
                            max(msz.height(), self.button_toggle.sizeHint().height() + frame_width * 2 + 2))

        self.button_toggle.pressed.connect(functools.partial(self.setText, ''))
        self.textChanged.connect(self.set_clear_button_visibility)
        self.button_toggle.setVisible(False)

    def set_clear_button_visibility(self, text):
        self.button_toggle.setVisible(False)
        if text:
            self.button_toggle.setVisible(True)

    def resizeEvent(self, event):
        sz = self.button_toggle.sizeHint()
        frame_width = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
        self.button_toggle.move(self.rect().right() - frame_width - sz.width(),
                           (self.rect().bottom() + 1 - sz.height())/2)



class ExtensionWidget(QFrame):

    open_tool = Signal(object)
    delete_tool = Signal(object)

    def __init__(self, data, *args, **kwargs):
        self.data = data
        self.extension_name = self.data['class_location'].split('.')[0]
        super(ExtensionWidget, self).__init__(*args, **kwargs)
        self.image_label = QLabel(self)
        self.text_label = QLabel(data['name'], self)
        self.body_label = QLabel('A tree veiw that lets you explore and create heirarchical scene graphs with nodes.', self)
        self.get_button = QPushButton('pip install...', self)
        self.busy_widget = ProgressWidget(self)

        self.main_vertical_layout = QVBoxLayout(self)
        self.top_horizontal_layout = QHBoxLayout()
        self.top_vertical_layout = QVBoxLayout()
        self.main_vertical_layout.addLayout(self.top_horizontal_layout)
        self.top_horizontal_layout.addStretch()
        self.top_horizontal_layout.addLayout(self.top_vertical_layout)
        self.top_horizontal_layout.addStretch()
        self.top_vertical_layout.addStretch()
        self.bottom_vertical_layout = QVBoxLayout()

        self.top_vertical_layout.addWidget(self.image_label)
        self.top_vertical_layout.addWidget(self.text_label)
        self.top_vertical_layout.addStretch()
        self.main_vertical_layout.addWidget(LineWidget(self))
        self.main_vertical_layout.addLayout(self.bottom_vertical_layout)
        self.bottom_vertical_layout.addStretch()
        self.bottom_vertical_layout.addWidget(self.body_label)
        self.bottom_vertical_layout.addStretch()
        self.bottom_vertical_layout.addWidget(self.get_button)
        self.text_label.setFont(QFont('', 12, False))
        self.text_label.setAlignment(Qt.AlignHCenter)
        self.image_label.setAlignment(Qt.AlignHCenter)
        self.body_label.setAlignment(Qt.AlignHCenter)


        image = QImage(med.get_icon_path(data['icon']))
        self.busy_widget.setAnimationDelay(70)
        self.busy_widget.setVisible(False)
        self.image_label.setPixmap(QPixmap.fromImage(image.scaled(45, 45)))
        self.body_label.setWordWrap(True)
        self.body_label.setFont(QFont('', 10, False))
        self.get_button.setFont(QFont('', 14, False))
        self.setProperty('WA_Hover', True)
        #self.get_button.setStyleSheet("background-color: grey;")
        self.main_vertical_layout.setSpacing(0)
        self.main_vertical_layout.setContentsMargins(5, 5, 5, 5)
        #self.bottom_vertical_layout.setSpacing(0)
        self.bottom_vertical_layout.setContentsMargins(5, 5, 5, 5)

        self.get_button.pressed.connect(self.download)

    def sizeHint(self, *args, **kwargs):
        return QSize(256, 256)

    def download(self):
        self.get_button.setText('installing')
        self.busy_widget.setVisible(True)
        self.busy_widget.startAnimation()
        self.tools_worker = FunctionWorker(pfn.pip_install, self.extension_name)
        self.tools_thread = QThread()
        self.tools_thread.start()
        self.tools_worker.moveToThread(self.tools_thread)
        self.tools_worker.start.connect(self.tools_worker.run)
        self.tools_worker.data.connect(self.new_extension)
        self.tools_worker.end.connect(self.finish_download)
        self.tools_worker.start.emit()

    def new_extension(self, package_name):
        print '---------->>', package_name
        current_user = env.user_handler.current_user
        current_user['settings']['extensions'].append(package_name)
        print 'EXTENSION WIDGET', current_user['settings']['extensions']
        env.user_handler.user_changed.emit(current_user)

    def finish_download(self):
        self.busy_widget.setVisible(False)
        self.busy_widget.stopAnimation()


        '''
        self.user_worker = FunctionWorker(
            uud.update_user_data,
            current_user['token'],
            extensions=users_extensions
        )
        self.user_thread = QThread()
        self.user_thread.start()
        self.user_worker.moveToThread(self.user_thread)

        #self.user_worker.end.connect(env.user_handler.set_user)
        self.user_worker.start.connect(self.user_worker.run)
        self.user_worker.start.emit()
        '''
        self.get_button.setText('pip uninstall...')



    def resizeEvent(self, event):
        self.busy_widget.resize(event.size())
        event.accept()



def clear_dir(path):
    shutil.rmtree(path, ignore_errors=False, onerror=onerror)


def onerror(func, path, exc_info):
    """
    Error handler for ``shutil.rmtree``.

    If the error is due to an access error (read only file)
    it attempts to add write permission and then retries.

    If the error is for another reason it re-raises the error.

    Usage : ``shutil.rmtree(path, onerror=onerror)``
    """
    import stat
    if not os.access(path, os.W_OK):
        # Is the error an access error ?
        os.chmod(path, stat.S_IWUSR)
        func(path)
    else:
        raise Exception('Could not delete path %s' % path)




class LineWidget(QFrame):
    def __init__(self, *args, **kwargs):
        super(LineWidget, self).__init__(*args, **kwargs)
        self.setFixedHeight(1)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.setStyleSheet("background-color: grey;")

if __name__ == '__main__':

    import sys
    import creaturecast_designer.stylesheets as stl
    app = QApplication(sys.argv)
    app.setStyleSheet(stl.get_stylesheet('slate'))
    mainWin = ExtensionsWidget()
    mainWin.resize(300, 500)
    mainWin.show()
    sys.exit(app.exec_())


import json
import time
import shutil
import requests
import os, zipfile, StringIO
from os.path import expanduser
import creaturecast_designer.services as svc
home_directory = expanduser("~").replace('\\', '/')
local_user = os.getenv('USER')
creaturecast_directory = '%screaturecast' % home_directory

def get_extension(name):
    get_extension_url = '%s/get_extension' % svc.library_url

    response = requests.post(
        get_extension_url,
        data=json.dumps(dict(name=name)),
        headers={'Content-Type': 'application/json'}
    )

    if response.status_code == 200:

        path = '%s/extensions/%s.zip' % (creaturecast_directory, name)
        dirname = os.path.dirname(path)
        if not os.path.exists(dirname):
            os.makedirs(dirname)

        with open(path, "wb") as f:
            f.write(response.content)
        print path
        print '%s/extensions' % creaturecast_directory
        zip_ref = zipfile.ZipFile(path, 'r')
        zip_ref.extractall('%s/extensions' % creaturecast_directory)
        zip_ref.close()
        return name

    else:
        print 'Failed with status code %s' % response.status_code



if __name__ == '__main__':
    print get_extension('creaturecast_scenegraph')


import os
import json
import shutil
from os.path import expanduser
import creaturecast_designer
import PySignal
home_directory = expanduser("~").replace('\\', '/')
local_user = os.getenv('USER')
creaturecast_directory = '%s/creaturecast' % home_directory
home_environment_variables_path = '%s/environment_variables.json' % creaturecast_directory
package_directory = os.path.dirname(creaturecast_designer.__file__.replace('\\', '/'))
environment_variables_path = '%s/data/environment_variables.json' % package_directory
modules_directory = '%s/modules' % creaturecast_directory

data_cache = dict()



class UserHandler(object):

    user_changed = PySignal.ClassSignal()

    def __init__(self, *args, **kwargs):
        super(UserHandler, self).__init__()
        self.current_user = None

    def set_user(self, user):
        self.current_user = user
        self.user_changed.emit(user)



class Settings(object):
    def __init__(self, *args, **kwargs):
        super(Settings, self).__init__()

    def __getitem__(self, item):
        return get_environment_variables()[item]

    def __setitem__(self, key, value):
        set_environment_variable(key, value)

    def get(self, *args):
        variables = get_environment_variables()
        if args[0] in variables:
            return variables[args[0]]
        elif len(args) > 1:
            return args[1]



user_handler = UserHandler()
settings = Settings()


def make_creaturecast_directory():
    if not os.path.exists(creaturecast_directory):
        os.makedirs(creaturecast_directory)

def make_environment_file():
    make_creaturecast_directory()
    if not os.path.exists(home_environment_variables_path):
        shutil.copyfile(environment_variables_path, home_environment_variables_path)

def get_environment_variables():
    make_environment_file()
    with open(home_environment_variables_path, mode='r') as f:
        return json.loads(f.read())

def set_environment_variable(key, value):
    variables = get_environment_variables()
    variables[key] = value
    with open(home_environment_variables_path, mode='w') as f:
        return f.write(json.dumps(variables))

def get_user_path(key):
    local_path = get_environment_variables().get(key, None)
    if not local_path:
        raise Exception('The user path "%s" was not found', key)
    return '%s%s' % (creaturecast_directory, local_path)


def get_data(key):
    if key in data_cache:
        return data_cache[key]
    home_path = copy_file_to_home_dir(key)
    with open(home_path, mode='r') as f:
        data = json.loads(f.read())
        data_cache[key] = data
        return data


def copy_file_to_home_dir(key):
    home_path = get_user_path(key)
    package_path = '%s/data/%s.json' % (package_directory, key)
    if not os.path.exists(home_path):
        shutil.copyfile(package_path, home_path)
    return home_path



library_url = 'http://creaturecast-library.herokuapp.com'
#library_url = 'http://127.0.0.1:5000'

def split_by_lines(text_generator):

    last_line = ""
    try:
        while True:
             chunk = "%s%s" % (last_line, next(text_generator))
             chunk_by_line = chunk.split('\n')
             last_line = chunk_by_line.pop()
             for line in chunk_by_line:
                 yield line

    except StopIteration: # the other end of the pipe is empty
        yield last_line
        raise StopIteration



import os
from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *

import creaturecast_designer.widgets.progress_indicator as prg
import creaturecast_designer.widgets.pixmap_label as pxl

class ConnectingWidget(QFrame):

    canceled = Signal()
    logged_in = Signal(object)
    font = QFont('', 9, False)
    button_font = QFont('', 12, False)

    def __init__(self, *args, **kwargs):
        super(ConnectingWidget, self).__init__(*args, **kwargs)

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self.creaturecast_label = pxl.PixmapLabel('/images/Creaturecast.png', self)
        self.creaturecast_label.setAlignment(Qt.AlignCenter)
        self.creaturecast_label.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)

        self.text_label = QLabel('Waiting for web login...', self)
        self.text_label.setAlignment(Qt.AlignCenter)
        self.text_label.setFont(self.font)
        self.cancel_button = QPushButton('Cancel', self)
        self.cancel_button.setFont(self.button_font)
        self.busy_widget = BusyWidget(self)


        self.vertical_layout = QVBoxLayout(self)
        self.horizontal_layout = QHBoxLayout()
        self.button_layout = QVBoxLayout()
        self.cancel_layout = QHBoxLayout()
        self.busy_layout = QHBoxLayout()

        self.busy_layout.addStretch()
        self.busy_layout.addWidget(self.busy_widget)
        self.busy_layout.addStretch()

        self.vertical_layout.addLayout(self.horizontal_layout)

        self.horizontal_layout.addLayout(self.button_layout)
        self.button_layout.addStretch()

        self.button_layout.addWidget(self.creaturecast_label)

        self.button_layout.addSpacerItem(QSpacerItem(20, 20))
        self.button_layout.addLayout(self.busy_layout)
        self.button_layout.addWidget(self.text_label)




        self.button_layout.addSpacerItem(QSpacerItem(20, 20))

        self.button_layout.addLayout(self.cancel_layout)

        self.cancel_layout.addStretch()
        self.cancel_layout.addWidget(self.cancel_button)
        self.cancel_layout.addStretch()
        self.button_layout.addStretch()
        self.button_layout.addStretch()

        self.cancel_button.pressed.connect(self.canceled.emit)

    def start_animation(self):
        self.busy_widget.startAnimation()




class BusyWidget(QWidget):

    def __init__(self, *args, **kwargs):
        super(BusyWidget, self).__init__(*args, **kwargs)

        # Initialize Qt Properties
        self.setProperties()

        # Intialize instance variables
        self.circumference = 64
        self.m_angle = 0
        self.m_timerId = -1
        self.m_delay = 40
        self.m_displayedWhenStopped = False
        self.m_color = Qt.white

        # Set size and focus policy
        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.setFocusPolicy(Qt.NoFocus)

        # Show the widget

    def animationDelay(self):
        return self.delay

    def isAnimated(self):
        return (self.m_timerId != -1)

    def isDisplayedWhenStopped(self):
        return self.displayedWhenStopped

    def getColor(self):
        return self.color

    def sizeHint(self):
        return QSize(self.circumference, self.circumference)

    def startAnimation(self):
        self.m_angle = 0

        if self.m_timerId == -1:
            self.m_timerId = self.startTimer(self.m_delay)

    def stopAnimation(self):
        if self.m_timerId != -1:
            self.killTimer(self.m_timerId)

        self.m_timerId = -1
        self.update()

    def setAnimationDelay(self, delay):
        if self.m_timerId != -1:
            self.killTimer(self.m_timerId)

        self.m_delay = delay

        if self.m_timerId != -1:
            self.m_timerId = self.startTimer(self.m_delay)

    def setDisplayedWhenStopped(self, state):
        self.displayedWhenStopped = state
        self.update()

    def setColor(self, color):
        self.m_color = color
        self.update()

    def timerEvent(self, event):
        self.m_angle = (self.m_angle + 30) % 360
        self.update()

    def paintEvent(self, event):
        if (not self.m_displayedWhenStopped) and (not self.isAnimated()):
            return

        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)

        outerRadius = self.circumference * 0.49
        innerRadius = self.circumference * 0.175

        capsuleHeight = outerRadius - innerRadius
        capsuleWidth = capsuleHeight * .25
        capsuleRadius = capsuleWidth / 2

        painter.setBrush(self.m_color)
        for i in range(0, 12):
            color = QColor(self.m_color)

            if self.isAnimated():
                color.setAlphaF(1.0 - (i / 12.0))
            else:
                color.setAlphaF(0.2)

            painter.setPen(Qt.NoPen)
            painter.setBrush(color)
            painter.save()
            center = self.rect().center()
            center.setY(float(center.y()))
            painter.translate(center)
            painter.rotate(self.m_angle - (i * 30.0))
            painter.drawRoundedRect(capsuleWidth * -0.5, (innerRadius + capsuleHeight) * -1, capsuleWidth,
                                    capsuleHeight, capsuleRadius, capsuleRadius)
            painter.restore()

    def setProperties(self):
        self.delay = Property(int, self.animationDelay, self.setAnimationDelay)
        self.displayedWhenStopped = Property(bool, self.isDisplayedWhenStopped, self.setDisplayedWhenStopped)
        self.color = Property(QColor, self.getColor, self.setColor)


import os

stylesheets_directory = os.path.dirname(__file__.replace('\\', '/'))


def get_stylesheet(stylesheet_name):
    with open('%s/%s.css' % (stylesheets_directory, stylesheet_name), mode='r') as f:
        return f.read()



from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *


class Connection(QtWidgets.QGraphicsPathItem):
    __defaultPen = QtGui.QPen(QtGui.QColor(168, 134, 3), 1.5)

    def __init__(self, graph, srcPortCircle, dstPortCircle):
        super(Connection, self).__init__()

        self.__graph = graph
        self.__srcPortCircle = srcPortCircle
        self.__dstPortCircle = dstPortCircle
        penStyle = QtCore.Qt.DashLine

        self.__connectionColor = QtGui.QColor(0, 0, 0)
        self.__connectionColor.setRgbF(*self.__srcPortCircle.getColor().getRgbF())
        self.__connectionColor.setAlpha(125)

        self.__defaultPen = QtGui.QPen(self.__connectionColor, 1.5, style=penStyle)
        self.__defaultPen.setDashPattern([1, 2, 2, 1])

        self.__connectionHoverColor = QtGui.QColor(0, 0, 0)
        self.__connectionHoverColor.setRgbF(*self.__srcPortCircle.getColor().getRgbF())
        self.__connectionHoverColor.setAlpha(255)

        self.__hoverPen = QtGui.QPen(self.__connectionHoverColor, 1.5, style=penStyle)
        self.__hoverPen.setDashPattern([1, 2, 2, 1])

        self.setPen(self.__defaultPen)
        self.setZValue(-1)

        self.setAcceptHoverEvents(True)
        self.connect()


    def setPenStyle(self, penStyle):
        self.__defaultPen.setStyle(penStyle)
        self.__hoverPen.setStyle(penStyle)
        self.setPen(self.__defaultPen) # Force a redraw


    def setPenWidth(self, width):
        self.__defaultPen.setWidthF(width)
        self.__hoverPen.setWidthF(width)
        self.setPen(self.__defaultPen) # Force a redraw


    def getSrcPortCircle(self):
        return self.__srcPortCircle


    def getDstPortCircle(self):
        return self.__dstPortCircle


    def getSrcPort(self):
        return self.__srcPortCircle.getPort()


    def getDstPort(self):
        return self.__dstPortCircle.getPort()


    def boundingRect(self):
        srcPoint = self.mapFromScene(self.__srcPortCircle.centerInSceneCoords())
        dstPoint = self.mapFromScene(self.__dstPortCircle.centerInSceneCoords())
        penWidth = self.__defaultPen.width()

        return QtCore.QRectF(
            min(srcPoint.x(), dstPoint.x()),
            min(srcPoint.y(), dstPoint.y()),
            abs(dstPoint.x() - srcPoint.x()),
            abs(dstPoint.y() - srcPoint.y()),
            ).adjusted(-penWidth/2, -penWidth/2, +penWidth/2, +penWidth/2)


    def paint(self, painter, option, widget):
        srcPoint = self.mapFromScene(self.__srcPortCircle.centerInSceneCoords())
        dstPoint = self.mapFromScene(self.__dstPortCircle.centerInSceneCoords())

        dist_between = dstPoint - srcPoint

        self.__path = QtGui.QPainterPath()
        self.__path.moveTo(srcPoint)
        self.__path.cubicTo(
            srcPoint + QtCore.QPointF(dist_between.x() * 0.4, 0),
            dstPoint - QtCore.QPointF(dist_between.x() * 0.4, 0),
            dstPoint
            )
        self.setPath(self.__path)
        super(Connection, self).paint(painter, option, widget)


    def hoverEnterEvent(self, event):
        self.setPen(self.__hoverPen)
        super(Connection, self).hoverEnterEvent(event)


    def hoverLeaveEvent(self, event):
        self.setPen(self.__defaultPen)
        super(Connection, self).hoverLeaveEvent(event)


    def mousePressEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.__dragging = True
            self._lastDragPoint = self.mapToScene(event.pos())
            event.accept()
        else:
            super(Connection, self).mousePressEvent(event)


    def mouseMoveEvent(self, event):
        if self.__dragging:
            pos = self.mapToScene(event.pos())
            delta = pos - self._lastDragPoint
            if delta.x() != 0:

                self.__graph.removeConnection(self)

                from . import mouse_grabber
                if delta.x() < 0:
                    mouse_grabber.MouseGrabber(self.__graph, pos, self.__srcPortCircle, 'In')
                else:
                    mouse_grabber.MouseGrabber(self.__graph, pos, self.__dstPortCircle, 'Out')

        else:
            super(Connection, self).mouseMoveEvent(event)


    def disconnect(self):
        self.__srcPortCircle.removeConnection(self)
        self.__dstPortCircle.removeConnection(self)


    def connect(self):
        self.__srcPortCircle.addConnection(self)
        self.__dstPortCircle.addConnection(self)



import os
import platform
import traceback
import time


from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *



class LoadingDialog(QDialog):
    def __init__(self, *args, **kwargs):
        super(LoadingDialog, self).__init__(*args, **kwargs)
        self.setWindowTitle('Connect Dialog')
        self.layout = QHBoxLayout(self)
        self.label = QLabel('Connecting...', self)
        self.label.setFont(QFont('Arial', 35, True))
        self.layout.addWidget(self.label)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_NoSystemBackground)
        self.setAttribute(Qt.WA_TranslucentBackground)
        #self.setStyleSheet("background-color:transparent;")

class RaisedFrame(QFrame):
    def __init__(self, *args, **kwargs):
        super(RaisedFrame, self).__init__(*args, **kwargs)
        self.setFrameStyle(QFrame.Panel)
        self.setAutoFillBackground(True)
        p = self.palette()
        p.setColor(self.backgroundRole(), QColor(80, 80, 80))
        self.setPalette(p)
        self.setFrameShadow(QFrame.Raised)


class TableView(QTableView):

    def __init__(self, *args, **kwargs):
        super(TableView, self).__init__(*args, **kwargs)
        self.items = args[0]

    def item_from_index(self, index):
        if index.row() != -1 and index.column() != -1:
            return self.items[index.row()][index.column()]



class ListView(QListView):

    def __init__(self, *args, **kwargs):
        super(ListView, self).__init__(*args, **kwargs)
        self.items = args[0]

class TreeView(QTreeView):

    def __init__(self, *args, **kwargs):
        super(TreeView, self).__init__(*args, **kwargs)


    def map_index_to_source(self, index):
        if isinstance(self.model(), QSortFilterProxyModel):
            index = self.model().mapToSource(index)
        return index

    def map_index_from_source(self, index):
        if isinstance(self.model(), QSortFilterProxyModel):
            index = self.model().mapFromSource(index)
        return index

    def get_model(self):
        model = self.model()
        while isinstance(model, QSortFilterProxyModel):
            model = model.sourceModel()
        return model

    def frame_node(self, index):

        index = self.map_index_from_source(index)
        self.clearSelection()
        self.set_expanded_descendants(QModelIndex(), False)
        self.set_expanded_parenpal
        self.scrollTo(index)

    def frame_nodes(self, indices):
        #self.clearSelection()
        #self.set_expanded_descendants(QModelIndex(), False)

        for index in indices:
            index = self.map_index_from_source(index)
            self.set_expanded_parents(index, True)
            self.select_item(index)
            self.scrollTo(index)


    def set_expanded_parents(self, index, value):
        index = self.map_index_from_source(index)

        model = self.get_model()
        for i in model.get_ancestors(index):
            self.setExpanded(i, value)

    def set_expanded_descendants(self, index, value):
        index = self.map_index_from_source(index)
        model = self.get_model()
        if model:
            for i in model.get_descendants(index):
                self.setExpanded(i, value)

    def select_item(self, index):
            self.selectionModel().select(
                index,
                QItemSelectionModel.Select |
                QItemSelectionModel.Rows
            )

class RatingBox(QComboBox):


    def __init__(self, data, key, *args, **kwargs):
        super(RatingBox, self).__init__(*args, **kwargs)
        self.data = data
        self.key = key
        self.setIconSize(QSize(55, 55))
        self.addItem(QIcon(mda.get_image_path('no_stars')), '')
        self.addItem(QIcon(mda.get_image_path('one_star')), '')
        self.addItem(QIcon(mda.get_image_path('two_stars')), '')
        self.addItem(QIcon(mda.get_image_path('three_stars')), '')
        self.addItem(QIcon(mda.get_image_path('four_stars')), '')
        self.addItem(QIcon(mda.get_image_path('five_stars')), '')
        self.setMaximumSize(QSize(88, 22))
        self.update_widget_value()
        self.currentIndexChanged.connect(self.set_data)

    def update_widget_value(self):
        if self.key in self.data:
            self.setCurrentIndex(self.data[self.key])
            self.setVisible(True)
        else:
            self.setVisible(False)

    def set_data(self, value):
        if self.key in self.data:
            self.setVisible(True)
            self.data[self.key] = value
        else:
            self.setVisible(False)


class Line(QFrame):
    def __init__(self, *args, **kwargs):
        super(Line, self).__init__(*args, **kwargs)
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)



class TightLayoutV(QVBoxLayout):
    def __init__(self, *args, **kwargs):
        super(TightLayoutV, self).__init__(*args, **kwargs)
        self.setSpacing(0)
        #self.setMargin(0)
        self.setContentsMargins(0, 0, 0, 0)

class TightLayoutH(QHBoxLayout):
    def __init__(self, *args, **kwargs):
        super(TightLayoutH, self).__init__(*args, **kwargs)
        self.setSpacing(0)
        #self.setMargin(0)
        self.setContentsMargins(0, 0, 0, 0)

class TightLayoutGrid(QGridLayout):
    def __init__(self, *args, **kwargs):
        super(TightLayoutGrid, self).__init__(*args, **kwargs)
        self.setSpacing(0)
        #self.setMargin(0)
        self.setContentsMargins(0, 0, 0, 0)

class IconButton(QPushButton):
    icon_changed = Signal(str)
    def __init__(self, icon_name, *args, **kwargs):
        super(IconButton, self).__init__(*args, **kwargs)
        self.icon_name = icon_name
        self.set_icon(icon_name)
        self.setFlat(True)
        self.setIconSize(QSize(64, 64))
        self.editable = False


    def mousePressEvent(self, event):
        if self.editable:
            if event.button() == Qt.LeftButton:
                path = mda.icon_directory
                if os.path.exists(path):
                    if platform.system() == 'Darwin':
                        #pySide and Mavericks needs special handling to start in correct dir
                        self.dialog = IconDialog(self, 'Pick an icon', r'%s/icons/icons_orig' % os.path.dirname(__file__))
                        self.dialog.selectFile(r'%s/icons/icons_orig/python.png' % os.path.dirname(__file__))
                        self.dialog.accepted.connect(self.set_icon_mac)
                        self.dialog.exec_()

                    else:
                        path, ok = IconDialog.getOpenFileName(self, 'Pick Icon', path, 'Png (*.png)')
                        if ok:
                            self.set_icon(os.path.basename(str(path)).split('.')[0])
                else:
                    raise IOError('Icon dir does not exist %s ' % path)

    def set_icon(self, icon_name):

        self.icon_name = icon_name

        super(IconButton, self).setIcon(QIcon(ico.get_icon(icon_name)))
        self.icon_changed.emit(self.icon_name)

    def set_icon_mac(self):
        sel = self.dialog.selectedFiles()
        if sel:
            self.set_icon(os.path.basename(sel[0]).split('.')[0])


class IconDialog(QFileDialog):
    def __init__(self, *args, **kwargs):
        super(IconDialog, self).__init__(*args, **kwargs)
        self.iconsDir = mda.icon_directory
        self.setDirectory(self.iconsDir)
        self.setFileMode(QFileDialog.ExistingFile)
        self.setViewMode(QFileDialog.List)
        self.directoryEntered.connect(self.blockDir)

    def blockDir(self, dir):
        if not dir == self.iconsDir:
            self.setDirectory(self.iconsDir)

def progress_generator(iterator, *args, **kwargs):
    prog=QProgressDialog(*args, **kwargs)
    prog.setModal(True)
    prog.show()
    prog.raise_()
    prog.setWindowTitle('Calculating...')
    prog.setWindowIcon(ico.get_icon('progress'))
    prog.setValue(0)
    itr = 0
    #try:
    for rig in iterator():
        #time.sleep(0.01)

        if prog.wasCanceled():
            break
        prog.setValue(itr)
        itr += 1
        yield rig

    #except:
    #print traceback.format_exc()
    #QMessageBox.warning(prog, 'Build Failed', '%s \n See Script Editor for details' % parse_error_message(traceback.format_exc()))
    prog.close()


    def error_dialog(self, e):
        if isinstance(e, prd.AuthenticationFailed):
            reply = QMessageBox.question(self, 'Authentication Failed', '%s\nWould you like to launch the license manager?' % e.message, QMessageBox.Yes, QMessageBox.No)
            if reply == QMessageBox.Yes:
                ui = lmg.LicenseManager(self)
                ui.show()
                #self.close()
        if isinstance(e, Exception):
            print traceback.format_exc()




def parse_error_message(errorString):
    items=errorString.split('\n')
    for itr, item in enumerate(items):
        if not item:
            return items[itr-1]
